-- C-One reconfigurable computer
-- http://c64upgra.de/c-one/
-- Subdesign C-One 
--
-- Copyright (c) 2005 Tobias Gubener
--
-- All rights reserved
--
-- Redistribution and use in source and synthezised forms, with or without
-- modification, are permitted provided that the following conditions are met:
--
-- Redistributions of source code must retain the above copyright notice,
-- this list of conditions and the following disclaimer.
--
-- Redistributions in synthesized form must reproduce the above copyright
-- notice, this list of conditions and the following disclaimer in the
-- documentation and/or other materials provided with the distribution.
--
-- Neither the name of the author nor the names of other contributors may
-- be used to endorse or promote products derived from this software without
-- specific prior written permission.
--
-- THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
-- AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
-- THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
-- PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
-- LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
-- CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
-- SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-- INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
-- CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
-- ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-- POSSIBILITY OF SUCH DAMAGE.
--



library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity crt is
port
	(
	data		: in std_logic_vector(7 downto 0);
	sysclk		: in std_logic;
	pixena		: in std_logic;
	reset		: in std_logic;
	videoadr	: out std_logic_vector(16 downto 0);
	video		: out std_logic_vector(15 downto 0);
	vsync		: buffer std_logic;
	hsync		: out std_logic
	);
end;

architecture rtl of crt is


signal 	vcount		: std_logic_vector(11 DOWNTO 0);
signal	hcount		: std_logic_vector(11 DOWNTO 0);
signal  addrcount	: std_logic_vector(16 DOWNTO 0);
signal	addrline	: std_logic_vector(16 DOWNTO 0);
signal	vhbyte		: std_logic_vector(7 DOWNTO 0);

begin
	videoadr<=addrcount;
		
	process (sysclk,reset) begin
		if reset='0' then
			vcount<=(others=>'0');
			hcount<=(others=>'0');
			addrcount<=(others=>'0');
			addrline<=(others=>'0');
		elsif (sysclk'event and sysclk='1') then
			if pixena='1' then
--Hsync			
				if hcount>654 and hcount<750 then
					hsync<='1';
				else
					hsync<='0';
				end if;		
					
--Vline count und Vsync				
				if hcount=655 then
					if vcount > 509-36 and vcount < 512-36 then
						vsync<='1';
					else
						vsync<='0';
					end if;
					if vcount<523 then
						vcount<=vcount+1;
					else
						vcount<=(others=>'0');
					end if;
				else
					vcount<=vcount;
					vsync<=vsync;	
				end if;

--Hline count 			
				if hcount<799 then
					hcount<=hcount+1;
					addrcount<=addrcount+1;
				else
					hcount<=(others=>'0');
					addrcount<=addrline;
				end if;
				
--Erzeugung des Videosignals				
				vhbyte<=data;
				if hcount<640 and vcount<408 then
					if hcount(0)='1' then
						video<=vhbyte & data;
					end if;
				else
					video<=(others=>'0');	
				end if;	
				
--Erzeugung der Adresse				
				if hcount=640 and vcount(0)='1' then
					addrline<=addrcount;
				elsif vcount=523 then
					addrline<=(others=>'0');
				end if;
							

				
			end if;
		end if;
	end process;
end;
